/*
 * Copyright (c) 1994, 1995.  Netscape Communications Corporation.  All
 * rights reserved.
 * 
 * Use of this software is governed by the terms of the license agreement for
 * the Netscape FastTrack or Netscape Enterprise Server between the
 * parties.
 */


/* ------------------------------------------------------------------------ */


/*
 * sem.h: Attempt to provide multi-process semaphores across platforms
 * 
 * Rob McCool
 */


#ifndef SEM_H
#define SEM_H

#include "systems.h"


/* All of the implementations currently use int as the semaphore type */
#ifdef SEM_WIN32
typedef HANDLE SEMAPHORE;
#define SEM_ERROR NULL
/* That oughta hold them (I hope) */
#define SEM_MAXVALUE 32767

#elif defined(SEM_IRIX)
#include <ulocks.h>
typedef struct {
    usptr_t *arena;
    usema_t *sem;
} semirix_s;
typedef semirix_s* SEMAPHORE;
#define SEM_ERROR NULL

#else /* ! SEM_WIN32, !SEM_IRIX */
typedef int SEMAPHORE;
#define SEM_ERROR -1
#endif /* SEM_WIN32 */

/*
 * sem_init creates a semaphore using the given name and unique 
 * identification number. filename should be a file accessible to the 
 * process. Returns SEM_ERROR on error.
 */

NSAPI_PUBLIC SEMAPHORE sem_init(char *name, int number);

/*
 * sem_terminate de-allocates the given semaphore.
 */

NSAPI_PUBLIC void sem_terminate(SEMAPHORE id);

/*
 * sem_grab attempts to gain exclusive access to the given semaphore. If
 * it can't get it, the caller will block. Returns -1 on error.
 */

NSAPI_PUBLIC int sem_grab(SEMAPHORE id);
NSAPI_PUBLIC int sem_tgrab(SEMAPHORE id);

/*
 * sem_release releases this process's exclusive control over the given
 * semaphore. Returns -1 on error.
 */

NSAPI_PUBLIC int sem_release(SEMAPHORE id);


#endif
